precision mediump float;

attribute highp vec2 a_texCoord;
attribute highp vec4 a_position;

varying vec2 v_texCoord;
varying vec4 v_color;

varying float v_brightness;

varying float v_dotCol;

varying highp vec2 v_dotTexCoord;

uniform mat4 MATRIX_MVP;
uniform float u_scale;

uniform vec2 u_resolution;

uniform highp vec4 u_time;

uniform float u_alpha;

const vec2 gridsize = vec2(39., 44.);   // was 23, 8

const int MAX_NUM_OF_TAPS = 2;
uniform highp vec3 u_touches[MAX_NUM_OF_TAPS];
const float duration = 1.;

const vec4 color2 = vec4(0., 254./255., 227./255., 1.);

const float M_PI_2 = 1.57079632679;
const float M_2_PI = 6.28318530718;

const float WIDTH_NORMAL_SQ = 141.1 * 1.44;      // 600. square   // was 20000.
const float RIPPLE_SPEED = 12.;     // was 52.

const vec2 VERT_FRACT = vec2(1.) / gridsize;

const vec4 C = vec4(0.211324865405187,  // (3.0-sqrt(3.0))/6.0
                        0.366025403784439,  // 0.5*(sqrt(3.0)-1.0)
                        -0.577350269189626,  // -1.0 + 2.0 * C.x
                        0.024390243902439); // 1.0 / 41.0


const float u_isGuide = 1.;


// Simplex Noise
vec3 mod289(vec3 x) {
    return x - floor(x * (1.0 / 289.0)) * 289.0;
}

vec2 mod289(vec2 x) {
    return x - floor(x * (1.0 / 289.0)) * 289.0;
}

vec3 permute(vec3 x) {
    return mod289(((x*34.0)+1.0)*x);
}

float snoise(highp vec2 v)
{
    // vec2 haha = v;

    
    // First corner
    vec2 i  = floor(v + dot(v, C.yy) );
    vec2 x0 = v -   i + dot(i, C.xx);

    // // float ifl = dot(v, vec2(0.366025403784439, 0.366025403784439));
    // float ifl = dot(haha, C.yy);

    // // vec2 i = vec2(dot(v, vec2(C.y, C.y)));
    // return ifl;
    
    // Other corners
    vec2 i1;
    float i1rf = max(sign(x0.x - x0.y), 0.0);
    i1 = vec2(i1rf, abs(i1rf - 1.0));
    vec4 x12 = x0.xyxy + C.xxzz;
    x12.xy -= i1;
    
    // Permutations
    i = mod289(i); // Avoid truncation effects in permutation
    vec3 p = permute( permute( i.y + vec3(0.0, i1.y, 1.0 ))
                     + i.x + vec3(0.0, i1.x, 1.0 ));
    
    vec3 m = max(0.5 - vec3(dot(x0,x0), dot(x12.xy,x12.xy), dot(x12.zw,x12.zw)), 0.0);
    m = m*m ;
    m = m*m ;
    
    vec3 x = 2.0 * fract(p * C.www) - 1.0;
    vec3 h = abs(x) - 0.5;
    vec3 ox = floor(x + 0.5);
    vec3 a0 = x - ox;
    
    // Normalise gradients implicitly by scaling m
    // Approximation of: m *= inversesqrt( a0*a0 + h*h );
    m *= 1.79284291400159 - 0.85373472095314 * ( a0*a0 + h*h );
    
    // Compute final noise value at P
    vec3 g;
    g.x  = a0.x  * x0.x  + h.x  * x0.y;
    g.yz = a0.yz * x12.xz + h.yz * x12.yw;
    return 130.0 * dot(m, g);
}




float quintEaseOut(float p)
{
    float f = p - 1.;
    return f * f * f * f * f + 1.;
}

float grad(float p)
{
    p = 1.0-abs(p-1.0);
    return p;
}

void main() {
    vec2 tc = a_texCoord;
    
    float xOffset;
    {
        float yidx = floor(tc.y / VERT_FRACT.y * 1.02);
        float yidxmod2 = mod(yidx, 2.);
        xOffset = u_resolution.x * VERT_FRACT.x * yidxmod2 * 0.5;
        
        float yidxd2 = floor(yidx / 2.);
        tc.x -= yidxd2 * VERT_FRACT.x;
    }
    
    v_texCoord = tc * gridsize;
    v_dotTexCoord = tc * 0.32 - vec2(u_time.x * 0.1, u_time.x * 0.5 * 0.1);
    
    float psizedelta = 0.;
    float rippledelta = 0.;

    vec2 MV_POS = a_position.xy * 1.1;
    MV_POS.x -= 40. * u_scale;

    
    for(int i=0; i<MAX_NUM_OF_TAPS; i++)
    {
        highp vec3 touch = u_touches[i] + vec3(0., -120. * u_scale, 0.);
        
        // PRESS
        float dist = distance(touch.xy, MV_POS.xy);
        
        float t = u_time.y - touch.z;
        t = (duration - t) / duration;
        t = max(0., t);
        t = quintEaseOut(1. - t);
        t = 1.-t;
        
        float a = smoothstep(0.,  smoothstep(0., 0.3, 1.-t) * 250. * u_scale, dist);
        a = 1. - a;
        a *= t;
        
        psizedelta += a;
        
        
//        // SPREAD
        highp vec2 pos_diff = touch.xy - MV_POS.xy;
        highp float distsq = pos_diff.x * pos_diff.x + pos_diff.y * pos_diff.y;

        float tapTime = u_time.y - touch.z;
        tapTime *= RIPPLE_SPEED;

        highp float wNormal = WIDTH_NORMAL_SQ * u_scale;
        highp float rippleDistNorm = distsq / (wNormal * wNormal);  // Normalize ripple distance
        tapTime = tapTime - 1.;
        rippleDistNorm *= 2.;
        rippleDistNorm = 1. - rippleDistNorm;
        rippleDistNorm += tapTime * u_scale;
        rippleDistNorm = grad(rippleDistNorm);
        rippledelta += max(rippleDistNorm, 0.) * t * 2.;
        
    }
    
    psizedelta = (min(1., psizedelta));
    
    float tcol = max(1. - tc.x * 1.1, 0.);
    
    float tcolv = smoothstep(0.95, 0.7, tc.y) * smoothstep(0., 0.15, tc.y);
    
    // vec2 ntc = tc * gridsize;
    // ntc.y *= 1.32;

    // float n = snoise(ntc * 0.06 - u_time.x * 0.2) * 0.5 + 0.5;      // was 0.13
    // float n = snoise(vec2(1., 1.));
    highp vec2 ntc = a_texCoord * 1.34;
    ntc.y *= 1.32;

    float n = snoise(ntc - u_time.x * 1.) * 0.5 + 0.5;
    float pn = n * n * n * n;
    float pn2 = n * n;
    
    // float dn = snoise(tc * 4. - u_time.x * 0.6);
    // float pdn = dn * dn * dn * dn;
    // v_dotCol = max(0., pdn * 5. * tcol) * tcolv;

    v_dotCol = tcol * tcolv * u_alpha;

    vec4 pos = a_position;    
    pos.x += xOffset;
    
    pos.z -= 200. * n ;     // was 200.
    pos.z -= (psizedelta) * 50. * (1. + pn * 1.);

    // pos.z -= psizedelta * 100.;
    
    // float shineC = sin(clamp(tc.x, 0., 1.) * M_2_PI - u_time.y * 1.6);
    // shineC = (shineC + 1.) * 0.5;
    // shineC = shineC * shineC * 2.2;

    // float shineC = sin(clamp(a_texCoord.x, 0., 1.) * M_2_PI - u_time.y * 3.);
    float shineC = sin(a_texCoord.x * 3. - a_texCoord.y  - u_time.y * 2.5) * 4. - 3.;
    shineC = max(0., shineC) * 1.;
    // shineC = shineC * shineC * 2.2;
    
    // v_brightness = 
    //     (max(abs((pos.z / 100.) * 0.8 + 0.2), 0.2) + 
    //     psizedelta * 0.8 + pn2 * 0.5) * 
    //     (tcol + psizedelta + rippledelta + shineC) * 0.55 * tcolv * u_alpha;

    float guideMultiplier = (1. - u_isGuide) * 5.;

    float peak = abs((pos.z / 200.) * 0.9 + 0.1);

    v_brightness = 
        (max(peak, 0.2) * 0.3 +        // ambience
        psizedelta * 0.8 + pn2 * 0.5) * 
        (tcol + psizedelta + rippledelta * 2.) * 0.55 * tcolv * u_alpha + shineC * (peak * 0.8 + 0.2);

    // v_brightness = shineC;
    
    gl_Position = MATRIX_MVP * pos;
}

