precision highp float;

const int MAX_NUM_OF_TAPS = 5;

attribute highp vec4 a_position;
attribute vec2 a_texCoord;

uniform mat4 MATRIX_MVP;
uniform vec2 u_resolution;
uniform float u_scale;

uniform highp vec3 u_touches[MAX_NUM_OF_TAPS];

uniform vec4 u_time;

varying vec4 v_color;
varying vec2 v_texCoord;
uniform float u_alpha;

const vec4 color1 = vec4(19./255., 86./255., 98./255., 1.);
const vec4 color2 = vec4(0., 254./255., 227./255., 1.);

const float duration = 1.;  // was 0.8
const float offset = 0.1;
const float offsetFr = offset / duration;   // was 0.1

float WAVE_POS(float time, float xpos)
{
    float _result = 1.0;

    _result *= sin(time * 0.5 + xpos * 0.003 / u_scale) * 3.168 * u_scale;
    _result *= sin(time * 1.3 + xpos * 0.007 / u_scale) * 6.048 * u_scale;

    return _result;
}

void main()
{
    vec4 pos = a_position;
    pos.y += WAVE_POS(u_time.y, pos.x);
    // pos.y -= a_texCoord.y * u_resolution.y;
    pos.y -= (a_texCoord.y-0.5) * 14.4 * u_scale;

    float glow = 0.;

    for(int i=0; i<MAX_NUM_OF_TAPS; i++)
    {
        float t = u_time.y - u_touches[i].z;
        t = (duration - t) / duration;
        t = max(0., t);

        highp float g = abs(u_touches[i].x - a_position.x);
        // g = smoothstep(0., 250. * u_scale, g);
        g = smoothstep(0., smoothstep(0., offsetFr, 1.-t) * 187. * u_scale, g);
        g = 1. - g;

        g *= t;

        glow += g * 1.9;
    }

    glow = min(glow, 2.5);
    
    v_texCoord = a_texCoord;
    v_color = u_alpha * mix(color1, color2, glow);
    
    gl_Position = MATRIX_MVP * pos;
}
