
precision mediump float;

// Dot Matrix shader
const float MIN_POINT_SIZE = 7.;    // 10.   7. 
const float MAX_POINT_SIZE = 30.;
//const float MAX_POINT_SIZE_DELTA = 16.;
const int MAX_NUM_OF_TAPS = 5;
const float WIDTH_NORMAL_SQ = 206.;      // 600. square
const float RIPPLE_SPEED = 20.;
const vec2 CENTER_POS = vec2(384. , 240.);   // Keyboard center position

attribute highp vec4 a_position;
attribute vec2 a_texCoord;

uniform mat4 MATRIX_MVP;
uniform highp vec3 u_touches[MAX_NUM_OF_TAPS];

uniform vec2 u_resolution;

uniform float u_scale;

varying vec4 v_color;

const float duration = 0.7;

uniform vec4 u_time;

const float M_PI_2 = 1.57079632679;
const float TIME_OFFSET = 0.05;
const float TIME_OFFSET_FRACT = TIME_OFFSET / duration;

uniform vec4 u_custom1; // x: size factor   y: z factor     z: alpha factor

//const vec4 color1 = vec4(255./255., 225./255., 231./255., 1.);
//const vec4 color2 = vec4(178./255., 243./255., 255./255., 1.);

float grad(float p)
{
   p = 1.0-abs(p-1.0);
   return p;
}

float quintEaseOut(float p)
{
    float f = p - 1.;
    return f * f * f * f * f + 1.;
}

float quadraticEaseInOut(float p)
{
    if(p < 0.5)
    {
        return 2. * p * p;
    }
    else
    {
        return (-2. * p * p) + (4. * p) - 1.;
    }
}

float elasticEaseOut(float p)
{
    return sin(-13. * M_PI_2 * (p + 1.)) * pow(2., -10. * p) + 1.;
}

void main()
{
    // Touch delta
    
    float psizedelta = 0.;
    vec2 xydelta = vec2(0.);
    
    
    
    for(int i=0; i<MAX_NUM_OF_TAPS; i++)
    {
        
        // PRESS
        float dist = distance(u_touches[i].xy, a_position.xy);
        vec2 normal = normalize(a_position.xy - u_touches[i].xy);

        float t = u_time.y - u_touches[i].z;
        t = (duration - t) / duration;
        t = max(0., t);
//        t = elasticEaseOut(1. - t);
        t = 1.-t;
        
        float t1 = smoothstep(0., TIME_OFFSET_FRACT, t);
        float t2 = smoothstep(TIME_OFFSET_FRACT, 1., t);
//        float s = elasticEaseOut(t2);
        float s = quintEaseOut(t2);
        s = 1. - s;
        
        float tr = mix(t1, s, step(TIME_OFFSET_FRACT, t));

        float a = smoothstep(0.,  288. * u_scale, dist);
        a = 1. - a;
        a *= tr;
        
        
        float b = smoothstep(0.,  184. * u_scale, dist);
        b = 1. - b;
        b *= tr;
        
//        xydelta += a * 60. * normal;
        
        psizedelta += a * 1.5 - b * 3.;

        
        
        // SPREAD
        
      // vec2 pos_diff = u_touches[i].xy - a_position.xy;
      // float distsq = pos_diff.x * pos_diff.x + pos_diff.y * pos_diff.y;
      
      // float tapTime = u_time.y - u_touches[i].z;
      // tapTime *= RIPPLE_SPEED;
      
      // float wNormal = WIDTH_NORMAL_SQ * u_scale;
      // float rippleDistNorm = distsq / (wNormal * wNormal);  // Normalize ripple distance

      // tapTime = tapTime - 1.;
      // rippleDistNorm *= 2.;
      // rippleDistNorm = 1. - rippleDistNorm;
      // rippleDistNorm += tapTime * u_scale;
      // rippleDistNorm = grad(rippleDistNorm);
      
      // psizedelta += max(rippleDistNorm, 0.) * t;


        
    }
     
    
    
    // Idle Noise
//    vec2 uv = a_texCoord;
//    uv *= 10.;
//    float idleNoise = snoise(vec2(uv.x + uv.y, u_time.y * 0.5));
//    idleNoise = idleNoise * 0.05 + 0.05;
    
    /*
    vec2 uv = a_texCoord * 1.5;
//    uv *= 1.5;
    float idleNoise = snoise(vec2(uv.x + uv.y * 0.5, u_time.y * 0.6));
    idleNoise = idleNoise * 0.05 + 0.05;
//    idleNoise *= (1. - smoothstep(0.3, 1., psizedelta));
    
    psizedelta += idleNoise;
    */
    
    psizedelta = min(1., psizedelta);
    
    // float psizedelta = clamp(rippleDistNorm * 16., 0., 16.);
    
    //    psizedelta = clamp(tapTime * 16. , 0., 16.);
    
    //    float psizedelta = clamp(16. - abs(distsq * 0.0002), 0., 16.);
    
    // Idle delta
    float idleDelta = 0.;
//    vec2 idle_diff = a_position.xy - CENTER_POS;
//    float idle_distsq = idle_diff.x * idle_diff.x + idle_diff.y * idle_diff.y;
//    float idle_dist = abs(sqrt(idle_distsq));

   float idle_dist = distance(abs(a_position.xy), u_resolution * vec2(0.5));
   idleDelta = sin(idle_dist / (100. * u_scale) - u_time.x * 11.) * 0.5 + 0.5 ;
    


    // vec2 idletc = a_texCoord.xy;
    // idletc.x = idletc.x * 1.6 - 0.3;
    
    // float idle_dist = distance(idletc.xy, vec2(0.5));
    // idleDelta = sin(idle_dist * 5. - u_time.x * 11.) * 0.5 + 0.5;

    //
    //    float actualDelta = mix(idleDelta, psizedelta, u_touchWeight);
    
    // gl_PointSize = max(MIN_POINT_SIZE + idleDelta * 13. + psizedelta * 30., MIN_POINT_SIZE) * u_scale * u_custom1.x;
    gl_PointSize = clamp(MIN_POINT_SIZE + idleDelta * 7. + psizedelta * 30., MIN_POINT_SIZE, MAX_POINT_SIZE) * u_scale;
//    gl_PointSize = max(MIN_POINT_SIZE + psizedelta * 30., MIN_POINT_SIZE) * u_scale * u_custom1.x;
    
    
//    gl_PointSize = 20.;
//    gl_PointSize = max(MIN_POINT_SIZE*2. - psizedelta, MIN_POINT_SIZE);
    
//    v_color = vec4(0.3 + 0.7 * psizedelta);   // was .15 .85
    
    // v_color = vec4(1. * u_custom1.z);
    v_color = vec4(1.);
    // v_color = vec4(1. - idle_dist * 0.0001, idle_dist * 0.0001, 0., 1.);
//    v_color = vec4(vec3(idleDelta, 0., 0.), 1.);
   // v_color = vec4(0.);
    
//    v_color = mix(vec4(1.), mix(color2, color1, step(375., a_position.x)), psizedelta);
    
//    v_color = vec4(a_texCoord, 1., 1.);
    
    vec4 pos = a_position;
//    pos.xy += xydelta;
   // pos.z += psizedelta * 90.;
//    pos.z += quadraticEaseInOut(psizedelta) * 220.;
//    pos.z += psizedelta * 160. * u_custom1.y;
    pos.z += idleDelta * 70. + psizedelta * 50.;    // was idle 60 psize 50
    
    gl_Position = MATRIX_MVP * pos;
}
